@@ -20,12 +20,16 @@ use libseccomp::ScmpSyscall;
 use nix::{
     errno::Errno,
     fcntl::{AtFlags, OFlag, OpenHow},
-    sys::socket::{bind, connect, Shutdown, SockFlag, SockaddrLike},
+    sys::socket::{bind, connect, AddressFamily, Shutdown, SockFlag, SockType, SockaddrLike},
     unistd::UnlinkatFlags,
     NixPath,
 };
 
-use crate::{compat::RenameFlags, path::XPath, rng::randint, uts::UtsName};
+use crate::{
+    compat::{MFdFlags, RenameFlags},
+    rng::randint,
+    uts::UtsName,
+};
 
 /// A platform-sized secure cookie
 ///
@@ -68,6 +72,14 @@ pub static SOCKET_COOKIE_ARG4: LazyLock<Cookie> = LazyLock::new(|| getcookie().e
 /// System call argument cookie: socket(2) $arg5
 #[expect(clippy::disallowed_methods)]
 pub static SOCKET_COOKIE_ARG5: LazyLock<Cookie> = LazyLock::new(|| getcookie().expect("getcookie"));
+/// System call argument cookie: socket(2) $arg4
+#[expect(clippy::disallowed_methods)]
+pub static SOCKETPAIR_COOKIE_ARG4: LazyLock<Cookie> =
+    LazyLock::new(|| getcookie().expect("getcookie"));
+/// System call argument cookie: socket(2) $arg5
+#[expect(clippy::disallowed_methods)]
+pub static SOCKETPAIR_COOKIE_ARG5: LazyLock<Cookie> =
+    LazyLock::new(|| getcookie().expect("getcookie"));
 /// System call argument cookie: accept4(2) $arg4
 #[expect(clippy::disallowed_methods)]
 pub static ACCEPT4_COOKIE_ARG4: LazyLock<Cookie> =
@@ -246,9 +258,9 @@ pub(crate) static PROCMAP_QUERY_COOKIE_ARG5: LazyLock<Cookie> =
 
 /// Safe openat2(2) confined by syscall cookies.
 #[inline(always)]
-pub(crate) fn safe_openat2<Fd: AsFd>(
+pub(crate) fn safe_openat2<Fd: AsFd, P: NixPath + ?Sized>(
     dirfd: Fd,
-    path: &XPath,
+    path: &P,
     mut how: OpenHow,
 ) -> Result<OwnedFd, Errno> {
     // SAFETY: In libc we trust.
@@ -320,6 +332,59 @@ pub fn safe_socket(
     })
 }
 
+/// socketpair(2) may be multiplexed by socketcall(2).
+pub static SYS_SOCKETPAIR: LazyLock<Option<libc::c_long>> = LazyLock::new(|| {
+    match ScmpSyscall::from_name("socketpair")
+        .map(i32::from)
+        .map(libc::c_long::from)
+        .ok()
+    {
+        Some(n) if n < 0 => None,
+        Some(n) => Some(n),
+        None => None,
+    }
+});
+
+/// Safe socketpair(2) confined by syscall cookies.
+#[inline(always)]
+pub fn safe_socketpair(
+    domain: AddressFamily,
+    ty: SockType,
+    proto: libc::c_int,
+    flags: SockFlag,
+) -> Result<(OwnedFd, OwnedFd), Errno> {
+    // Merge SockType and SockFlags together.
+    let mut ty = ty as libc::c_int;
+    ty |= flags.bits();
+
+    let mut fds = [-1, -1];
+
+    if let Some(sys_socketpair) = *SYS_SOCKETPAIR {
+        // SAFETY: In libc we trust.
+        Errno::result(unsafe {
+            libc::syscall(
+                sys_socketpair,
+                domain as libc::c_int,
+                ty,
+                proto,
+                fds.as_mut_ptr(),
+                *SOCKETPAIR_COOKIE_ARG4,
+                *SOCKETPAIR_COOKIE_ARG5,
+            )
+        })?;
+    } else {
+        // SAFETY:
+        // socketcall(2) on multiplexed architecture.
+        // We use libc version for convenience.
+        Errno::result(unsafe {
+            libc::socketpair(domain as libc::c_int, ty, proto, fds.as_mut_ptr())
+        })?;
+    }
+
+    // SAFETY: socketpair returns valid FDs on success.
+    unsafe { Ok((OwnedFd::from_raw_fd(fds[0]), OwnedFd::from_raw_fd(fds[1]))) }
+}
+
 /// accept4(2) may be multiplexed by socketcall(2).
 pub static SYS_ACCEPT4: LazyLock<Option<libc::c_long>> = LazyLock::new(|| {
     match ScmpSyscall::from_name("accept4")
@@ -501,34 +566,43 @@ pub fn safe_shutdown<Fd: AsFd>(fd: Fd, how: Shutdown) -> Result<(), Errno> {
 
 /// Safe memfd_create(2) confined by syscall cookies.
 #[inline(always)]
-pub(crate) fn safe_memfd_create(name: &[u8], flags: libc::c_uint) -> Result<OwnedFd, Errno> {
-    // SAFETY: In libc we trust.
+pub fn safe_memfd_create<P: NixPath + ?Sized>(name: &P, flags: MFdFlags) -> Result<OwnedFd, Errno> {
+    // name limit is 249 bytes, excluding the terminating null byte.
+    if name.len() > 249 {
+        return Err(Errno::EINVAL);
+    }
+
+    let res = name.with_nix_path(|cstr| {
+        // SAFETY: In libc we trust.
+        unsafe {
+            libc::syscall(
+                libc::SYS_memfd_create,
+                cstr.as_ptr(),
+                flags.bits(),
+                *MEMFD_CREATE_COOKIE_ARG2,
+                *MEMFD_CREATE_COOKIE_ARG3,
+                *MEMFD_CREATE_COOKIE_ARG4,
+                *MEMFD_CREATE_COOKIE_ARG5,
+            )
+        }
+    })?;
+
     #[expect(clippy::cast_possible_truncation)]
-    let fd = Errno::result(unsafe {
-        libc::syscall(
-            libc::SYS_memfd_create,
-            name.as_ptr(),
-            flags,
-            *MEMFD_CREATE_COOKIE_ARG2,
-            *MEMFD_CREATE_COOKIE_ARG3,
-            *MEMFD_CREATE_COOKIE_ARG4,
-            *MEMFD_CREATE_COOKIE_ARG5,
-        )
-    })? as RawFd;
+    let fd = Errno::result(res).map(|fd| fd as RawFd)?;
 
     // SAFETY:
     //
-    // `memfd_create(2)` should return a valid owned fd on success
+    // `memfd_create(2)` returns a valid owned fd on success.
     Ok(unsafe { OwnedFd::from_raw_fd(fd) })
 }
 
 /// Safe renameat2(2) confined by syscall cookies.
 #[inline(always)]
-pub(crate) fn safe_renameat2<Fd1: AsFd, Fd2: AsFd>(
+pub(crate) fn safe_renameat2<Fd1: AsFd, Fd2: AsFd, P1: NixPath + ?Sized, P2: NixPath + ?Sized>(
     old_dirfd: Fd1,
-    old_path: &XPath,
+    old_path: &P1,
     new_dirfd: Fd2,
-    new_path: &XPath,
+    new_path: &P2,
     flags: RenameFlags,
 ) -> Result<(), Errno> {
     let res = old_path.with_nix_path(|old_cstr| {
@@ -603,7 +677,7 @@ static SYS_FTRUNCATE64: LazyLock<Option<libc::c_long>> = LazyLock::new(|| {
 });
 
 /// Safe truncate(2) confined by syscall cookies.
-pub(crate) fn safe_truncate(path: &XPath, len: libc::off_t) -> Result<(), Errno> {
+pub(crate) fn safe_truncate<P: NixPath + ?Sized>(path: &P, len: libc::off_t) -> Result<(), Errno> {
     // On ILP32 where off_t is 64-bit, dispatch to truncate64 ABI.
     if size_of::<libc::off_t>() > size_of::<libc::c_long>() {
         return safe_truncate64(path, libc::off64_t::from(len));
@@ -628,7 +702,10 @@ pub(crate) fn safe_truncate(path: &XPath, len: libc::off_t) -> Result<(), Errno>
 }
 
 /// Safe truncate64(2) confined by syscall cookies.
-pub(crate) fn safe_truncate64(path: &XPath, len: libc::off64_t) -> Result<(), Errno> {
+pub(crate) fn safe_truncate64<P: NixPath + ?Sized>(
+    path: &P,
+    len: libc::off64_t,
+) -> Result<(), Errno> {
     #[cfg(not(any(
         target_pointer_width = "64",
         all(target_arch = "x86_64", target_pointer_width = "32"),
@@ -809,9 +886,9 @@ pub(crate) fn safe_ftruncate64<Fd: AsFd>(fd: Fd, len: libc::off64_t) -> Result<(
 
 /// Safe unlinkat(2) confined by syscall cookies.
 #[inline(always)]
-pub(crate) fn safe_unlinkat<Fd: AsFd>(
+pub(crate) fn safe_unlinkat<Fd: AsFd, P: NixPath + ?Sized>(
     dirfd: Fd,
-    path: &XPath,
+    path: &P,
     flag: UnlinkatFlags,
 ) -> Result<(), Errno> {
     let atflag = match flag {
@@ -837,11 +914,11 @@ pub(crate) fn safe_unlinkat<Fd: AsFd>(
 
 /// Safe linkat(2) confined by syscall cookies.
 #[inline(always)]
-pub(crate) fn safe_linkat<Fd1: AsFd, Fd2: AsFd>(
+pub(crate) fn safe_linkat<Fd1: AsFd, Fd2: AsFd, P1: NixPath + ?Sized, P2: NixPath + ?Sized>(
     olddirfd: Fd1,
-    oldpath: &XPath,
+    oldpath: &P1,
     newdirfd: Fd2,
-    newpath: &XPath,
+    newpath: &P2,
     flag: AtFlags,
 ) -> Result<(), Errno> {
     let res = oldpath.with_nix_path(|oldcstr| {
