@@ -24,9 +24,9 @@ use crate::{
     config::PROC_FILE,
     cookie::safe_linkat,
     fs::{FileType, FsFlags},
-    hook::{PathArgs, SysArg, SysFlags, UNotifyEventRequest},
     kernel::{syscall_path_handler, to_atflags},
     path::{XPath, XPathBuf},
+    req::{PathArgs, SysArg, SysFlags, UNotifyEventRequest},
 };
 
 pub(crate) fn sys_link(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -105,9 +105,9 @@ fn syscall_link_handler(
     restrict_hardlinks: bool,
 ) -> Result<ScmpNotifResp, Errno> {
     // SAFETY: SysArg has two elements.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let old_path = args.0.as_ref().unwrap();
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let new_path = args.1.as_ref().unwrap();
 
     // SAFETY: Using AT_EMPTY_PATH requires CAP_DAC_READ_SEARCH
