@@ -27,7 +27,7 @@ use crate::{
     fs::{fd_status_flags, safe_open_msym, CanonicalPath},
     kernel::sandbox_path,
     path::XPathBuf,
-    proc::{proc_mem_limit, proc_statm},
+    proc::{proc_mem, proc_statm},
     req::UNotifyEventRequest,
     sandbox::{Action, Capability, IntegrityError},
     warn,
@@ -457,45 +457,42 @@ fn syscall_mem_handler(request: UNotifyEventRequest, syscall: MemSyscall) -> Scm
 
     // Check PSS
     if mem_max > 0 {
-        match proc_mem_limit(req.pid(), mem_max) {
-            Ok(false) => {
-                // SAFETY: No pointer dereference in security check.
-                unsafe { request.continue_syscall() }
-            }
-            Ok(true) => {
-                if mem_act != Action::Filter {
-                    if verbose {
-                        warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
-                            "sys": &name, "mem_max": mem_max,
-                            "tip": "increase `mem/max'",
-                            "req": &request);
-                    } else {
-                        warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
-                            "sys": &name, "mem_max": mem_max,
-                            "tip": "increase `mem/max'",
-                            "pid": request.scmpreq.pid);
-                    }
-                }
-                match mem_act {
-                    // Allow cannot happen.
-                    Action::Warn => {
-                        // SAFETY: No pointer dereference in security check.
-                        unsafe { request.continue_syscall() }
-                    }
-                    Action::Deny | Action::Filter => request.fail_syscall(Errno::ENOMEM),
-                    Action::Panic => panic!(),
-                    Action::Exit => std::process::exit(libc::ENOMEM),
-                    _ => {
-                        // Stop|Kill
-                        let _ = request.kill(mem_act);
-                        request.fail_syscall(Errno::ENOMEM)
-                    }
+        let mem_cur = match proc_mem(req.pid()) {
+            Ok(mem_cur) => mem_cur,
+            Err(errno) => return request.fail_syscall(errno),
+        };
+        if mem_cur >= mem_max {
+            if mem_act != Action::Filter {
+                if verbose {
+                    warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
+                        "sys": &name, "mem_max": mem_max, "mem_cur": mem_cur,
+                        "tip": "increase `mem/max'",
+                        "req": &request);
+                } else {
+                    warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
+                        "sys": &name, "mem_max": mem_max, "mem_cur": mem_cur,
+                        "tip": "increase `mem/max'",
+                        "pid": request.scmpreq.pid);
                 }
             }
-            Err(errno) => request.fail_syscall(errno),
+            return match mem_act {
+                // Allow cannot happen.
+                Action::Warn => {
+                    // SAFETY: No pointer dereference in security check.
+                    unsafe { request.continue_syscall() }
+                }
+                Action::Deny | Action::Filter => request.fail_syscall(Errno::ENOMEM),
+                Action::Panic => panic!(),
+                Action::Exit => std::process::exit(libc::ENOMEM),
+                _ => {
+                    // Stop|Kill
+                    let _ = request.kill(mem_act);
+                    request.fail_syscall(Errno::ENOMEM)
+                }
+            };
         }
-    } else {
-        // SAFETY: No pointer dereference in security check.
-        unsafe { request.continue_syscall() }
     }
+
+    // SAFETY: No pointer dereference in security check.
+    unsafe { request.continue_syscall() }
 }
