@@ -30,19 +30,21 @@ use crate::{
         SEE_EMILY_PLAY, SYD_EL, THE_PIPER,
     },
     confine::EIDRM,
-    cookie::safe_openat2,
+    cookie::{safe_memfd_create, safe_openat2},
     err::err2no,
     error,
     fs::{
-        create_memfd, format_oflags, is_dev_kfd, lock_fd, retry_on_eintr, safe_open_how_msym,
-        safe_open_msym, seal_memfd, unlock_fd, CanonicalPath, FileInfo, FileType, FsFlags, MaybeFd,
+        format_oflags, is_dev_kfd, oflag_accmode, oflag_nonblock, safe_open_how_msym,
+        safe_open_msym, seal_memfd, CanonicalPath, FileInfo, FileType, FsFlags, MaybeFd,
     },
     hash::aes_ctr_tmp,
-    hook::{SysArg, UNotifyEventRequest},
     info, log_enabled,
     mask::mask_proc_pid_status,
+    ofd::{lock_fd, unlock_fd},
     path::{XPath, XPathBuf},
     proc::{proc_tty, proc_umask},
+    req::{SysArg, UNotifyEventRequest},
+    retry::retry_on_eintr,
     sandbox::{Action, Capability},
     syslog::LogLevel,
     warn,
@@ -203,21 +205,14 @@ fn syscall_open_handler(
 
         // Determine capabilities based on access mode:
         //
-        // 1. glibc does not include O_PATH to O_ACCMODE.
-        // 2. musl defines O_PATH equal to O_EXEC and O_SEARCH,
-        //    and O_ACCMODE is defined as O_ACCMODE|O_SEARCH.
-        // Here we force the second behaviour by explicitly
-        // adding O_PATH into O_ACCMODE. This works on both libcs.
-        // See: https://www.openwall.com/lists/musl/2013/02/22/1
-        //
         // TODO: Confine O_PATH file descriptors with Stat category,
         // rather than Read category when they can be properly emulated.
         // See: https://bugzilla.kernel.org/show_bug.cgi?id=218501
-        let (mut caps, o_path, o_rdwr) = match flags.bits() & (libc::O_ACCMODE | libc::O_PATH) {
-            libc::O_RDONLY => (Capability::CAP_READ, false, false),
-            libc::O_WRONLY => (Capability::CAP_WRITE, false, false),
-            libc::O_RDWR => (Capability::CAP_READ | Capability::CAP_WRITE, false, true),
-            libc::O_PATH => (Capability::CAP_READ, true, false),
+        let (mut caps, o_path, o_rdwr) = match oflag_accmode(flags) {
+            OFlag::O_RDONLY => (Capability::CAP_READ, false, false),
+            OFlag::O_WRONLY => (Capability::CAP_WRITE, false, false),
+            OFlag::O_RDWR => (Capability::CAP_READ | Capability::CAP_WRITE, false, true),
+            OFlag::O_PATH => (Capability::CAP_READ, true, false),
             _ => return Err(Errno::EINVAL), // Invalid access mode.
         };
 
@@ -287,7 +282,7 @@ fn syscall_open_handler(
         }
 
         // Read the remote path and check for magic path.
-        let (mut path, magic) = request.read_path(&sandbox, arg, true)?;
+        let (mut path, magic, _) = request.read_path(&sandbox, arg, true)?;
 
         if sandbox.verbose {
             crate::debug!("ctx": "open", "op": "read_path",
@@ -316,8 +311,7 @@ fn syscall_open_handler(
             }
 
             let fd = if caps.contains(Capability::CAP_READ) {
-                let fd = create_memfd(b"syd-box\0", *SAFE_MFD_FLAGS)?;
-                let mut file = File::from(fd);
+                let mut file = safe_memfd_create(c"syd-box", *SAFE_MFD_FLAGS).map(File::from)?;
 
                 const FLEN: usize = MAGIC_PREFIX.len() + 3; /* .{el,sh} */
                 let fack = path.abs().len() == FLEN;
@@ -345,7 +339,7 @@ fn syscall_open_handler(
                 // SAFETY: Deny further writes to the file descriptor.
                 seal_memfd(&file)?;
 
-                MaybeFd::Owned(OwnedFd::from(file))
+                MaybeFd::Owned(file.into())
             } else {
                 MaybeFd::RawFd(NULL_FD())
             };
@@ -408,27 +402,15 @@ fn syscall_open_handler(
                     }
 
                     // This may or may not block, better safe than sorry.
-                    (
-                        false,
-                        !(flags.contains(OFlag::O_NONBLOCK) || flags.contains(OFlag::O_NDELAY)),
-                        false,
-                    )
+                    (false, !oflag_nonblock(flags), false)
                 }
                 FileType::Lnk => {
                     // This may or may not block, better safe than sorry.
-                    (
-                        false,
-                        !(flags.contains(OFlag::O_NONBLOCK) || flags.contains(OFlag::O_NDELAY)),
-                        false,
-                    )
+                    (false, !oflag_nonblock(flags), false)
                 }
                 FileType::Chr | FileType::Fifo | FileType::Sock => {
                     // Character devices, fifos and sockets may block.
-                    (
-                        false,
-                        !(flags.contains(OFlag::O_NONBLOCK) || flags.contains(OFlag::O_NDELAY)),
-                        false,
-                    )
+                    (false, !oflag_nonblock(flags), false)
                 }
                 FileType::Blk => {
                     // SAFETY:
@@ -440,11 +422,7 @@ fn syscall_open_handler(
                     }
 
                     // Block devices may block.
-                    (
-                        false,
-                        !(flags.contains(OFlag::O_NONBLOCK) || flags.contains(OFlag::O_NDELAY)),
-                        false,
-                    )
+                    (false, !oflag_nonblock(flags), false)
                 }
                 FileType::Unk => {
                     // SAFETY:
@@ -921,7 +899,7 @@ fn syscall_open_handler(
 
                         // Open a memory fd to be sealed for write.
                         // Memory fd is exec-sealed on Linux>=6.3.
-                        let mut mfd = create_memfd(b"syd-machine-id\0", *SAFE_MFD_FLAGS)
+                        let mut mfd = safe_memfd_create(c"syd-machine-id", *SAFE_MFD_FLAGS)
                             .map(File::from)
                             .map(BufWriter::new)?;
 
@@ -941,7 +919,7 @@ fn syscall_open_handler(
                         //
                         // Open a memory fd to be sealed for write.
                         // Memory fd is exec-sealed on Linux>=6.3.
-                        let mut mfd = create_memfd(b"syd-proc-pid-status\0", *SAFE_MFD_FLAGS)
+                        let mut mfd = safe_memfd_create(c"syd-proc-pid-status", *SAFE_MFD_FLAGS)
                             .map(File::from)?;
 
                         // Filter proc_pid_status(5) into the memory fd.
@@ -1089,7 +1067,7 @@ fn handle_crypt(
     //
     // Similarly, `aes_ctr_tmp` respects non-blocking opens too,
     // and returns a non-blocking fd as necessary.
-    let wait = !(safe_flags.contains(OFlag::O_NONBLOCK) || safe_flags.contains(OFlag::O_NDELAY));
+    let wait = !oflag_nonblock(safe_flags);
     retry_on_eintr(|| lock_fd(&enc_fd, true, create || wait))?;
 
     // Acquire encryption sockets.
