@@ -21,9 +21,9 @@ use crate::{
     confine::{scmp_arch_bits, EOWNERDEAD},
     fs::{parse_fd, CanonicalPath, FileInfo, FileType, FsFlags},
     hash::SydHashSet,
-    hook::{SysArg, SysFlags, UNotifyEventRequest},
     kernel::{sandbox_path, to_atflags},
     path::XPath,
+    req::{SysArg, SysFlags, UNotifyEventRequest},
     sandbox::Capability,
 };
 
@@ -225,7 +225,6 @@ fn syscall_stat_handler(
         let req = request.scmpreq;
         let is_fd = arg.path.is_none();
         let sandbox = request.get_sandbox();
-        let is_lock = sandbox.locked_for(req.pid());
 
         let has_crypt = sandbox.enabled(Capability::CAP_CRYPT);
         let has_stat = sandbox.enabled(Capability::CAP_STAT);
@@ -235,14 +234,17 @@ fn syscall_stat_handler(
             return Err(if is_fd { Errno::EACCES } else { Errno::ENOENT });
         }
 
-        // Read the remote path.
-        // If lock is on do not check for magic path.
-        let (mut path, magic) = request.read_path(&sandbox, arg, !is_lock)?;
+        // Read the remote path and check for magic path.
+        let (mut path, magic, _) = request.read_path(&sandbox, arg, true)?;
 
         let restrict_stat_bdev = !sandbox.flags.allow_unsafe_stat_bdev();
         let restrict_stat_cdev = !sandbox.flags.allow_unsafe_stat_cdev();
         let mut ghost = false;
-        if !is_lock && magic {
+        if magic {
+            if sandbox.locked_for(req.pid()) {
+                // Sandbox is locked, access denied.
+                return Err(Errno::ENOENT);
+            }
             drop(sandbox); // release the read-lock.
 
             // Handle magic prefix (ie /dev/syd)
