@@ -1,6 +1,9 @@
+// SPDX-License-Identifier: Apache-2.0 OR MIT
+
 #[cfg(test)]
 use std::convert::TryInto;
 
+use bitflags::bitflags;
 #[cfg(test)]
 use strum::{EnumCount, IntoEnumIterator};
 #[cfg(test)]
@@ -61,8 +64,12 @@ pub enum ABI {
     /// Fifth Landlock ABI, introduced with
     /// [Linux 6.10](https://git.kernel.org/stable/c/2fc0e7892c10734c1b7c613ef04836d57d4676d5).
     V5 = 5,
-    /// Sixth Landlock ABI, introduced with Linux 6.12.
+    /// Sixth Landlock ABI, introduced with
+    /// [Linux 6.12](https://git.kernel.org/stable/c/e1b061b444fb01c237838f0d8238653afe6a8094).
     V6 = 6,
+    /// Seventh Landlock ABI, introduced with
+    /// [Linux 6.15](https://git.kernel.org/stable/c/72885116069abdd05c245707c3989fc605632970).
+    V7 = 7,
 }
 
 impl ABI {
@@ -99,8 +106,9 @@ impl From<i32> for ABI {
             3 => ABI::V3,
             4 => ABI::V4,
             5 => ABI::V5,
+            6 => ABI::V6,
             // Returns the greatest known ABI.
-            _ => ABI::V6,
+            _ => ABI::V7,
         }
     }
 }
@@ -139,20 +147,21 @@ fn known_abi() {
 }
 
 #[cfg(test)]
-lazy_static! {
-    static ref TEST_ABI: ABI = match std::env::var("LANDLOCK_CRATE_TEST_ABI") {
-        Ok(s) => {
-            let n = s.parse::<i32>().unwrap();
+static TEST_ABI: std::sync::LazyLock<ABI> = std::sync::LazyLock::new(|| {
+    use std::os::unix::ffi::OsStrExt;
+
+    match std::env::var_os("LANDLOCK_CRATE_TEST_ABI") {
+        Some(s) => {
+            let n = btoi::btoi::<i32>(s.as_bytes()).expect("LANDLOCK_CRATE_TEST_ABI");
             if ABI::is_known(n) || n == 0 {
                 ABI::from(n)
             } else {
                 panic!("Unknown ABI: {n}");
             }
         }
-        Err(std::env::VarError::NotPresent) => ABI::new_current(),
-        Err(e) => panic!("Failed to read LANDLOCK_CRATE_TEST_ABI: {e}"),
-    };
-}
+        None => ABI::new_current(),
+    }
+});
 
 #[cfg(test)]
 pub(crate) fn can_emulate(mock: ABI, partial_support: ABI, full_support: Option<ABI>) -> bool {
@@ -188,7 +197,16 @@ fn current_kernel_abi() {
     // to the Landlock ABI version supported by your kernel.  With a missing variable, the latest
     // Landlock ABI version known by this crate is automatically set.
     // From Linux 5.13 to 5.18, you need to run: LANDLOCK_CRATE_TEST_ABI=1 cargo test
-    assert_eq!(*TEST_ABI, ABI::new_current());
+    let test_abi = *TEST_ABI;
+    let current_abi = ABI::new_current();
+    println!(
+        "Current kernel version: {}",
+        std::fs::read_to_string("/proc/version")
+            .unwrap_or_else(|_| "unknown".into())
+            .trim()
+    );
+    println!("Expected Landlock ABI {test_abi:?} whereas the current ABI is {current_abi:#?}");
+    assert_eq!(test_abi, current_abi);
 }
 
 // CompatState is not public outside this crate.
@@ -281,7 +299,6 @@ impl From<ABI> for Compatibility {
 
 impl Compatibility {
     // Compatibility is a semi-opaque struct.
-    #[allow(clippy::new_without_default)]
     pub(crate) fn new() -> Self {
         ABI::new_current().into()
     }
@@ -388,7 +405,7 @@ pub trait Compatible: Sized + private::OptionCompatLevelMut {
     ///         // However, this ruleset may also handle other (future) access rights
     ///         // if they are supported by the running kernel.
     ///         .set_compatibility(CompatLevel::BestEffort)
-    ///         .handle_access(AccessFs::from_all(ABI::V5))?
+    ///         .handle_access(AccessFs::from_all(ABI::V6))?
     ///         .create()?)
     /// }
     /// ```
@@ -417,7 +434,7 @@ pub trait Compatible: Sized + private::OptionCompatLevelMut {
     ///         // if they are supported by the running kernel,
     ///         // but without returning any error otherwise.
     ///         .set_compatibility(CompatLevel::BestEffort)
-    ///         .handle_access(AccessFs::from_all(ABI::V5))?
+    ///         .handle_access(AccessFs::from_all(ABI::V6))?
     ///         .create()?)
     /// }
     /// ```
@@ -444,7 +461,7 @@ pub trait Compatible: Sized + private::OptionCompatLevelMut {
 }
 
 #[test]
-#[allow(deprecated)]
+#[expect(deprecated)]
 fn deprecated_set_best_effort() {
     use crate::landlock::{CompatLevel, Compatible, Ruleset};
 
@@ -665,3 +682,63 @@ where
         }
     }
 }
+
+bitflags! {
+    /// Kernel "errata" reported by `landlock_create_ruleset(NULL, 0, ERRATA)`.
+    ///
+    /// The errata interface returns a bitmask of user-visible fixes that may
+    /// relax or clarify behavior. Bits are **1-based** in kernel docs, i.e.,
+    /// "Erratum 1" corresponds to bit 0 in this mask.
+    ///
+    /// Known errata:
+    ///
+    /// - **TCP socket identification (Erratum 1, tied to ABI v4):**
+    ///   Fixes non-TCP stream protocols (e.g., SMC/MPTCP/SCTP) being gated by
+    ///   TCP rights during `bind(2)`/`connect(2)`. With this bit set, only TCP
+    ///   sockets are governed by TCP access rights.
+    /// - **Scoped signal handling (Erratum 2, tied to ABI v6):**
+    ///   Always allow signals between threads of the same process (same
+    ///   thread-group), even with signal scoping enabled. This matches how
+    ///   user-space updates credentials across threads and mirrors ptrace rules.
+    ///
+    /// Notes:
+    /// - The mask is **runtime** feature detection and may be backported.
+    /// - Absence of a bit means the fix is not present on this kernel.
+    /// - This is orthogonal to the Landlock ABI version probe.
+    ///
+    /// See: `landlock_create_ruleset(2)` and `landlock(7)`.
+    #[derive(Clone, Copy, Debug, Default, Eq, PartialEq, Hash)]
+    pub struct Errata: u32 {
+        /// Erratum 1: TCP socket identification (impacts ABI v4).
+        ///
+        /// Ensures only TCP sockets are controlled by TCP access rights; other
+        /// stream protocols are unaffected by TCP rules.
+        const TCP_SOCKET_IDENTIFICATION = 1 << 0;
+
+        /// Erratum 2: Scoped signal handling (impacts ABI v6).
+        ///
+        /// Always allow signals between threads of the same process,
+        /// independent of Landlock signal scoping.
+        const SCOPED_SIGNAL_SAME_TGID   = 1 << 1;
+    }
+}
+
+impl Errata {
+    /// Query the running kernel for enabled errata and return them as flags.
+    ///
+    /// On kernels without Landlock or without the errata probe, this returns
+    /// `Ok(ErrataFlags::empty())`.
+    ///
+    /// Unknown errata flags are retained.
+    pub fn query() -> Self {
+        match unsafe {
+            // Landlock ABI version starts at 1 but errno is only set for negative values.
+            uapi::landlock_create_ruleset(std::ptr::null(), 0, uapi::LANDLOCK_CREATE_RULESET_ERRATA)
+        } {
+            -1 => Self::empty(),
+            n => u32::try_from(n)
+                .map(Self::from_bits_retain)
+                .unwrap_or(Self::empty()),
+        }
+    }
+}
