@@ -25,7 +25,7 @@ use std::{
 
 use btoi::btoi;
 use memchr::{
-    arch::all::{is_equal, is_prefix, is_suffix, memchr::One},
+    arch::all::{is_equal, is_prefix, is_suffix},
     memchr, memmem, memrchr,
 };
 use nix::{
@@ -406,16 +406,19 @@ impl XPath {
             return Ok(());
         };
         let proc_pid = if let Some(pid) = proc_pid {
-            pid
+            Pid::from_raw(pid)
         } else {
             return Ok(());
         };
+        if proc_pid == pid {
+            return Ok(());
+        }
 
         //
         // RESTRICTION 2: Protect readdir(/proc).
         //
         // SAFETY: Prevent /proc process tree traversal.
-        if is_proc && proc_pid != pid.as_raw() {
+        if is_proc {
             return Err(Errno::ENOENT);
         }
         // END OF RESTRICTION 2
@@ -427,7 +430,6 @@ impl XPath {
         //
         // Step 1: Protect Syd thread group.
         let syd_pid = Pid::this();
-        let proc_pid = Pid::from_raw(proc_pid);
         if proc_pid == syd_pid {
             return Err(Errno::ENOENT);
         }
@@ -944,41 +946,39 @@ impl XPath {
     /// Checks if the path consists only of "." components.
     pub fn is_dot(&self) -> bool {
         let bytes = self.as_bytes();
-
         if bytes.is_empty() {
             return false;
         }
 
-        let mut has_component = false;
-        let mut start = 0;
+        let mut i = 0;
+        let mut saw_component = false;
 
-        // Iterate over '/' positions without allocating.
         #[expect(clippy::arithmetic_side_effects)]
-        for sep in One::new(b'/').iter(bytes) {
-            if sep > start {
-                // Non-empty component [start, sep).
-                let component = &bytes[start..sep];
-                has_component = true;
-                if component != b"." {
-                    return false;
-                }
+        while i < bytes.len() {
+            // Skip one or more '/'.
+            while i < bytes.len() && bytes[i] == b'/' {
+                i += 1;
+            }
+            if i == bytes.len() {
+                break; // trailing slashes only
             }
 
-            // Move past '/'.
-            start = sep + 1;
-        }
+            // Start of a non-empty component.
+            saw_component = true;
 
-        // Handle the trailing component after the last '/',
-        // or the whole slice if none.
-        if start < bytes.len() {
-            let component = &bytes[start..];
-            has_component = true;
-            if component != b"." {
+            // Component must be exactly "."
+            if bytes[i] != b'.' {
                 return false;
             }
+            i += 1;
+
+            // If the component continues, it must immediately be a '/' (or end).
+            if i < bytes.len() && bytes[i] != b'/' {
+                return false; // e.g., ".." or ".hidden"
+            }
         }
 
-        has_component
+        saw_component
     }
 
     /// Determine whether path is equal to the given string.
@@ -1674,10 +1674,10 @@ mod tests {
 
     #[test]
     fn test_path_check_file_type() {
-        assert!(XPathBuf::from("/proc")
+        xpath!("/proc")
             .check(Pid::from_raw(1), Some(&FileType::Dir), None, true, true)
-            .is_ok());
-        assert!(XPathBuf::from("/proc")
+            .unwrap();
+        xpath!("/proc")
             .check(
                 Pid::from_raw(1),
                 Some(&FileType::Dir),
@@ -1685,8 +1685,8 @@ mod tests {
                 true,
                 true,
             )
-            .is_ok());
-        assert!(XPathBuf::from("/proc")
+            .unwrap();
+        xpath!("/proc")
             .check(
                 Pid::from_raw(1),
                 Some(&FileType::Reg),
@@ -1694,38 +1694,38 @@ mod tests {
                 true,
                 true,
             )
-            .is_ok());
-        assert!(XPathBuf::from("/dev/null")
+            .unwrap();
+        xpath!("/dev/null")
             .check(Pid::from_raw(1), Some(&FileType::Chr), None, true, true)
-            .is_ok());
-        assert!(XPathBuf::from("/dev/log")
+            .unwrap();
+        xpath!("/dev/log")
             .check(Pid::from_raw(1), Some(&FileType::Sock), None, true, true)
-            .is_ok());
-        assert!(XPathBuf::from("/dev/fifo")
+            .unwrap();
+        xpath!("/dev/fifo")
             .check(Pid::from_raw(1), Some(&FileType::Fifo), None, true, true)
-            .is_ok());
-        assert!(XPathBuf::from("/dev/sda1")
+            .unwrap();
+        xpath!("/dev/sda1")
             .check(Pid::from_raw(1), Some(&FileType::Blk), None, true, true)
-            .is_err());
-        assert!(XPathBuf::from("/dev/lmao")
+            .unwrap_err();
+        xpath!("/dev/lmao")
             .check(Pid::from_raw(1), Some(&FileType::Unk), None, true, true)
-            .is_err());
+            .unwrap_err();
     }
 
     #[test]
     fn test_path_check_procfs() {
         let this = Pid::from_raw(128);
         let that = Pid::from_raw(256);
-        assert!(XPathBuf::from("/proc")
+        xpath!("/proc")
             .check(
                 this,
                 Some(&FileType::Dir),
                 Some(&xpath!("{this}")),
                 true,
-                true
+                true,
             )
-            .is_ok());
-        assert!(XPathBuf::from(format!("/proc/{this}"))
+            .unwrap();
+        xpath!("/proc/{this}")
             .check(
                 this,
                 Some(&FileType::Reg),
@@ -1733,8 +1733,8 @@ mod tests {
                 true,
                 true,
             )
-            .is_ok());
-        assert!(XPathBuf::from(format!("/proc/{this}"))
+            .unwrap();
+        xpath!("/proc/{this}")
             .check(
                 this,
                 Some(&FileType::Dir),
@@ -1742,26 +1742,26 @@ mod tests {
                 true,
                 true,
             )
-            .is_ok());
-        assert!(XPathBuf::from(format!("/proc/{this}/task"))
+            .unwrap();
+        xpath!("/proc/{this}/task")
             .check(
                 this,
                 Some(&FileType::Dir),
                 Some(&xpath!("{this}")),
                 true,
-                true
+                true,
             )
-            .is_ok());
-        assert!(XPathBuf::from("/proc")
+            .unwrap();
+        xpath!("/proc")
             .check(
                 this,
                 Some(&FileType::Dir),
                 Some(&xpath!("{that}")),
                 true,
-                true
+                true,
             )
-            .is_err());
-        assert!(XPathBuf::from(format!("/proc/{that}"))
+            .unwrap_err();
+        xpath!("/proc/{that}")
             .check(
                 this,
                 Some(&FileType::Reg),
@@ -1769,8 +1769,8 @@ mod tests {
                 true,
                 true,
             )
-            .is_ok());
-        assert!(XPathBuf::from(format!("/proc/{that}"))
+            .unwrap();
+        xpath!("/proc/{that}")
             .check(
                 this,
                 Some(&FileType::Dir),
@@ -1778,31 +1778,41 @@ mod tests {
                 true,
                 true,
             )
-            .is_ok());
-        assert!(XPathBuf::from(format!("/proc/{that}/task"))
+            .unwrap();
+        xpath!("/proc/{that}/task")
             .check(
                 this,
                 Some(&FileType::Dir),
                 Some(&xpath!("{that}")),
                 true,
-                true
+                true,
             )
-            .is_ok());
+            .unwrap();
     }
 
     #[test]
     fn test_path_check_procfs_syd_leader() {
         let syd = Pid::this();
-        assert!(XPathBuf::from("/proc")
+        let pid = Pid::from_raw(syd.as_raw() + 1);
+        xpath!("/proc")
             .check(
                 syd,
                 Some(&FileType::Dir),
                 Some(&xpath!("{syd}")),
                 true,
-                true
+                true,
             )
-            .is_err());
-        assert!(XPathBuf::from(format!("/proc/{syd}"))
+            .unwrap();
+        xpath!("/proc")
+            .check(
+                pid,
+                Some(&FileType::Dir),
+                Some(&xpath!("{syd}")),
+                true,
+                true,
+            )
+            .unwrap_err();
+        xpath!("/proc/{syd}")
             .check(
                 syd,
                 Some(&FileType::Reg),
@@ -1810,30 +1820,49 @@ mod tests {
                 true,
                 true,
             )
-            .is_err());
-        assert!(XPathBuf::from(format!("/proc/{syd}"))
+            .unwrap();
+        xpath!("/proc/{syd}")
             .check(
-                syd,
+                pid,
+                Some(&FileType::Reg),
+                Some(&XPath::from_bytes(b"")),
+                true,
+                true,
+            )
+            .unwrap_err();
+        xpath!("/proc/{syd}")
+            .check(
+                pid,
                 Some(&FileType::Dir),
                 Some(&XPath::from_bytes(b"")),
                 true,
                 true,
             )
-            .is_err());
-        assert!(XPathBuf::from(format!("/proc/{syd}/task"))
+            .unwrap_err();
+        xpath!("/proc/{syd}/task")
             .check(
                 syd,
                 Some(&FileType::Dir),
                 Some(&xpath!("{syd}")),
                 true,
-                true
+                true,
             )
-            .is_err());
+            .unwrap();
+        xpath!("/proc/{syd}/task")
+            .check(
+                pid,
+                Some(&FileType::Dir),
+                Some(&xpath!("{syd}")),
+                true,
+                true,
+            )
+            .unwrap_err();
     }
 
     #[test]
     fn test_path_check_procfs_syd_thread() {
         // Spawn a new thread.
+        let pid = Pid::this();
         let tid = {
             let (tx, rx) = mpsc::channel();
             thread::spawn(move || {
@@ -1842,16 +1871,16 @@ mod tests {
             });
             rx.recv().unwrap()
         };
-        assert!(XPathBuf::from("/proc")
+        xpath!("/proc")
             .check(
                 tid,
                 Some(&FileType::Dir),
                 Some(&xpath!("{tid}")),
                 true,
-                true
+                true,
             )
-            .is_err());
-        assert!(XPathBuf::from(format!("/proc/{tid}"))
+            .unwrap();
+        xpath!("/proc/{tid}")
             .check(
                 tid,
                 Some(&FileType::Reg),
@@ -1859,8 +1888,8 @@ mod tests {
                 true,
                 true,
             )
-            .is_err());
-        assert!(XPathBuf::from(format!("/proc/{tid}"))
+            .unwrap();
+        xpath!("/proc/{tid}")
             .check(
                 tid,
                 Some(&FileType::Dir),
@@ -1868,16 +1897,52 @@ mod tests {
                 true,
                 true,
             )
-            .is_err());
-        assert!(XPathBuf::from(format!("/proc/{tid}/task"))
+            .unwrap();
+        xpath!("/proc/{tid}/task")
             .check(
                 tid,
                 Some(&FileType::Dir),
                 Some(&xpath!("{tid}")),
                 true,
-                true
+                true,
             )
-            .is_err());
+            .unwrap();
+        xpath!("/proc")
+            .check(
+                pid,
+                Some(&FileType::Dir),
+                Some(&xpath!("{tid}")),
+                true,
+                true,
+            )
+            .unwrap_err();
+        xpath!("/proc/{tid}")
+            .check(
+                pid,
+                Some(&FileType::Reg),
+                Some(&XPath::from_bytes(b"")),
+                true,
+                true,
+            )
+            .unwrap_err();
+        xpath!("/proc/{tid}")
+            .check(
+                pid,
+                Some(&FileType::Dir),
+                Some(&XPath::from_bytes(b"")),
+                true,
+                true,
+            )
+            .unwrap_err();
+        xpath!("/proc/{tid}/task")
+            .check(
+                pid,
+                Some(&FileType::Dir),
+                Some(&xpath!("{tid}")),
+                true,
+                true,
+            )
+            .unwrap_err();
     }
 
     #[test]
