@@ -56,7 +56,9 @@ use nix::{
 };
 use procfs_core::{
     net::UnixNetEntries,
-    process::{LimitValue, MMPermissions, MMapPath, MemoryMaps, Namespace, Namespaces},
+    process::{
+        LimitValue, MMPermissions, MMapPath, MemoryMaps, Namespace, Namespaces, SmapsRollup,
+    },
     CryptoTable, FromBufRead,
 };
 
@@ -736,6 +738,45 @@ pub fn proc_smaps(pid: Pid) -> Result<Vec<SydMemoryMap>, Errno> {
         .map_err(proc_errno)
 }
 
+/// Return the rollup memory map of the given process.
+///
+/// This is more efficient than using `proc_smaps` and iterating over maps.
+/// See: https://www.kernel.org/doc/Documentation/ABI/testing/procfs-smaps_rollup
+pub fn proc_smaps_rollup(pid: Pid) -> Result<SydMemoryMap, Errno> {
+    let mut path = XPathBuf::from_pid(pid);
+    path.push(b"smaps_rollup");
+
+    // Safely open proc(5) file.
+    let reader = safe_open_proc(&path)
+        .map(File::from)
+        .map(BufReader::new)
+        .map_err(proc_errno)?;
+
+    SmapsRollup::from_buf_read(reader)
+        .map_err(|err| proc_error_to_errno(&err).unwrap_or(Errno::ENOSYS))
+        .map_err(proc_errno)
+        .and_then(|maps| {
+            maps.memory_map_rollup
+                .into_iter()
+                .next()
+                .map(SydMemoryMap)
+                .ok_or(Errno::ENOMEM)
+        })
+}
+
+/// Calculate total memory usage of a process using _proc_pid_smaps_rollup(5).
+///
+/// See: https://www.kernel.org/doc/Documentation/ABI/testing/procfs-smaps_rollup
+pub fn proc_mem(pid: Pid) -> Result<u64, Errno> {
+    let map = proc_smaps_rollup(pid)?;
+    let mut sum = 0u64;
+    for key in ["Pss", "Private_Dirty", "Shared_Dirty"] {
+        let val = map.0.extension.map.get(key).copied().unwrap_or(0);
+        sum = sum.saturating_add(val);
+    }
+    Ok(sum)
+}
+
 /// Returns the memory map of an address by reading _proc_pid_maps_(5).
 ///
 /// Returns EFAULT if the address does not fall within any range.
@@ -1034,67 +1075,17 @@ pub fn proc_task_limit(pid: Pid, max: u64) -> Result<bool, Errno> {
     Ok(false)
 }
 
-/// Checks whether process memory usage is within the give maximum.
-///
-/// This function uses the `procfs` crate to obtain detailed memory maps
-/// from `/proc/[pid]/smaps`. It sums multiple memory usage values reported in these maps
-/// to calculate a more comprehensive total memory usage.
-///
-/// # Returns
-///
-/// This function returns a `Result<bool, Errno>`. It returns Ok(true)
-/// if the limit was exceeded Ok(false) otherwise. On failure, it
-/// returns `Errno`.
-///
-/// # Errors
-///
-/// This function returns an error if it fails to retrieve the process's memory maps,
-/// typically due to insufficient permissions or an invalid process ID.
-pub fn proc_mem_limit(pid: Pid, max: u64) -> Result<bool, Errno> {
-    match proc_smaps(pid) {
-        Ok(maps) => {
-            let mut total_size: u64 = 0;
-            for map in &maps {
-                match &map.0.pathname {
-                    MMapPath::Path(_)
-                    | MMapPath::Anonymous
-                    | MMapPath::Stack
-                    | MMapPath::Other(_) => {
-                        let pss = map.0.extension.map.get("Pss").copied().unwrap_or(0);
-                        let private_dirty = map
-                            .0
-                            .extension
-                            .map
-                            .get("Private_Dirty")
-                            .copied()
-                            .unwrap_or(0);
-                        let shared_dirty = map
-                            .0
-                            .extension
-                            .map
-                            .get("Shared_Dirty")
-                            .copied()
-                            .unwrap_or(0);
-
-                        total_size = total_size.saturating_add(
-                            pss.saturating_add(private_dirty)
-                                .saturating_add(shared_dirty),
-                        );
-
-                        // Stop processing if total size exceeds or equals max
-                        if total_size >= max {
-                            return Ok(true);
-                        }
-                    }
-                    _ => (),
-                }
-            }
-
-            // If we're at this point, we did not hit the limit.
-            Ok(false)
-        }
-        Err(_) => Err(Errno::last()),
-    }
+/// Represents an executable file record from proc_pid_maps(5).
+#[derive(Clone, Debug, Eq, PartialEq, Ord, PartialOrd, Hash)]
+pub struct SydExecMap {
+    /// Path to the executable
+    pub path: XPathBuf,
+    /// Inode of the executable
+    pub inode: u64,
+    /// Major device ID of the executable
+    pub dev_major: i32,
+    /// Minor device ID of the executable
+    pub dev_minor: i32,
 }
 
 /// Collects all unique paths with executable permissions from the
@@ -1102,12 +1093,11 @@ pub fn proc_mem_limit(pid: Pid, max: u64) -> Result<bool, Errno> {
 ///
 /// # Returns
 ///
-/// A `Result` containing a vector of `(XPathBuf,u64,i32,i32)` objects
+/// A `Result` containing a vector of `SydExecMap` objects
 /// for all unique paths with executable permissions along with their
 /// device ID and inode. On error, it returns an `Errno`. If no executable
 /// is found in maps, returns `Errno::ENOEXEC`.
-#[expect(clippy::type_complexity)]
-pub fn proc_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i32)>, Errno> {
+pub fn proc_executables(pid: Pid) -> Result<Vec<SydExecMap>, Errno> {
     if *crate::config::HAVE_PROCMAP_QUERY {
         // Use the efficient PROCMAP_QUERY ioctl(2) on Linux>=6.11.
         return procmap_query_executables(pid);
@@ -1117,7 +1107,7 @@ pub fn proc_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i32)>, Errn
 
     // Create a SydIndexSet to store unique executable paths while
     // preserving insertion order.
-    let mut paths = SydIndexSet::default();
+    let mut set = SydIndexSet::default();
 
     // Iterate over each memory map entry.
     for map in maps {
@@ -1126,18 +1116,24 @@ pub fn proc_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i32)>, Errn
             // Check if the map has executable permissions.
             if map.0.perms.contains(MMPermissions::EXECUTE) {
                 // Insert the path into the SydIndexSet.
-                paths.insert((path.into(), map.0.inode, map.0.dev.0, map.0.dev.1));
+                let exe = SydExecMap {
+                    path: path.into(),
+                    inode: map.0.inode,
+                    dev_major: map.0.dev.0,
+                    dev_minor: map.0.dev.1,
+                };
+                set.insert(exe);
             }
         }
     }
 
     // Check for no match.
-    if paths.is_empty() {
+    if set.is_empty() {
         return Err(Errno::ENOEXEC);
     }
 
     // Convert the SydIndexSet to a vector and return.
-    Ok(paths.into_iter().collect())
+    Ok(set.into_iter().collect())
 }
 
 /// Collects all unique paths with executable permissions from the
@@ -1145,12 +1141,11 @@ pub fn proc_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i32)>, Errn
 ///
 /// # Returns
 ///
-/// A `Result` containing a vector of `(XPathBuf, u64, i32, i32)` objects
+/// A `Result` containing a vector of `SydExecMap` objects
 /// for all unique paths with executable permissions along with their
 /// inode and device numbers. On error, returns an `Errno`. If no executable
 /// mappings are found, returns `Errno::ENOEXEC`.
-#[expect(clippy::type_complexity)]
-pub fn procmap_query_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i32)>, Errno> {
+pub fn procmap_query_executables(pid: Pid) -> Result<Vec<SydExecMap>, Errno> {
     // Open /proc/$pid/maps.
     let maps = proc_maps_open(pid)?;
 
@@ -1161,7 +1156,7 @@ pub fn procmap_query_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i3
 
     let mut path = [0u8; PATH_MAX];
     let mut addr = 0u64;
-    let mut paths = SydIndexSet::default();
+    let mut set = SydIndexSet::default();
 
     loop {
         match procmap_query(&maps, flags, addr, Some(&mut path), None) {
@@ -1178,7 +1173,13 @@ pub fn procmap_query_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i3
                     .map(XPathBuf::from)
                     .map_err(|_| Errno::EINVAL)?;
                 #[expect(clippy::cast_possible_wrap)]
-                paths.insert((path, q.inode, q.dev_major as i32, q.dev_minor as i32));
+                let exe = SydExecMap {
+                    path,
+                    inode: q.inode,
+                    dev_major: q.dev_major as i32,
+                    dev_minor: q.dev_minor as i32,
+                };
+                set.insert(exe);
 
                 addr = q.vma_end;
             }
@@ -1188,12 +1189,12 @@ pub fn procmap_query_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i3
     }
 
     // Check for no match.
-    if paths.is_empty() {
+    if set.is_empty() {
         return Err(Errno::ENOEXEC);
     }
 
     // Convert the SydIndexSet to a vector and return.
-    Ok(paths.into_iter().collect())
+    Ok(set.into_iter().collect())
 }
 
 /// This data structure represents a Virtual Memory Area.
@@ -1350,6 +1351,8 @@ pub fn proc_namespaces(pid: Pid) -> Result<Namespaces, Errno> {
     // Read directory entries using getdents64 in a loop.
     #[expect(clippy::disallowed_types)]
     let mut namespaces = std::collections::HashMap::new();
+    let mut seen_dot = false;
+    let mut seen_dotdot = false;
     loop {
         let mut entries = match getdents64(&dirfd, DIRENT_BUF_SIZE) {
             Ok(iter) => iter,
@@ -1358,7 +1361,12 @@ pub fn proc_namespaces(pid: Pid) -> Result<Namespaces, Errno> {
         };
 
         for entry in &mut entries {
-            if entry.is_dot() {
+            if !seen_dot && entry.is_dot() {
+                seen_dot = true;
+                continue;
+            }
+            if !seen_dotdot && entry.is_dotdot() {
+                seen_dotdot = true;
                 continue;
             }
 
@@ -2369,15 +2377,16 @@ mod tests {
         }
 
         // Ensure uniqueness and preserved order.
-        let list = proc_executables(Pid::this()).unwrap();
+        let bins = proc_executables(Pid::this()).unwrap();
         let mut seen = SydIndexSet::default();
-        for (path, _, _, _) in &list {
+        for bin in &bins {
+            let path = &bin.path;
             assert!(seen.insert(path.clone()), "duplicate path {path}!");
         }
 
         // Check that insertion order matches returned order.
         let collected: Vec<_> = seen.into_iter().collect();
-        let returned: Vec<_> = list.iter().map(|(p, _, _, _)| p.clone()).collect();
+        let returned: Vec<_> = bins.iter().map(|bin| bin.path.clone()).collect();
         assert_eq!(collected, returned);
     }
 
@@ -2388,8 +2397,9 @@ mod tests {
         }
 
         // Returned paths should exist on filesystem and have exec bit.
-        let list = proc_executables(Pid::this()).unwrap();
-        for (path, _, _, _) in list {
+        let bins = proc_executables(Pid::this()).unwrap();
+        for bin in bins {
+            let path = bin.path;
             let md = std::fs::metadata(&path).expect("path does not exist");
             let perms = md.permissions().mode();
             assert!(
