@@ -356,16 +356,7 @@ impl RemoteProcess {
                     None
                 };
 
-                let path =
-                    safe_canonicalize(self.pid, fd, &path, arg.fsflags, Some(sandbox.deref()))?;
-
-                // Validate request after canonicalize which may
-                // have involved a procfs(5) read.
-                if request.map(|req| !req.is_valid()).unwrap_or(false) {
-                    return Err(Errno::ESRCH);
-                }
-
-                path
+                safe_canonicalize(self.pid, fd, &path, arg.fsflags, Some(sandbox.deref()))?
             }
         } else {
             // SAFETY: SysArg.path is None asserting dirfd is Some.
@@ -586,27 +577,16 @@ impl RemoteProcess {
         addr: u64,
         request: Option<&UNotifyEventRequest>,
     ) -> Result<XPathBuf, Errno> {
-        // Initialize path on the heap,
-        // bail out if memory allocation fails.
-        let mut buf = Vec::new();
-        buf.try_reserve(PATH_MIN).or(Err(Errno::ENOMEM))?;
+        // Initialize path on the stack.
+        let mut buf = [0u8; PATH_MAX];
 
         // Read from process memory.
-        // We read PATH_MIN bytes at a time,
-        // because most paths are short.
+        // We read PATH_MIN bytes at a time, because most paths are short.
         let mut off = 0;
         #[expect(clippy::arithmetic_side_effects)]
         while off < PATH_MAX {
-            // Ensure we have enough space for the next read.
+            // Prepare slice to read.
             let len = PATH_MIN.min(PATH_MAX - off);
-
-            if buf.len() < off + len {
-                // Extend the buffer to the required size,
-                // bail out if memory allocation fails.
-                buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
-                buf.resize(off + len, 0);
-            }
-
             let ptr = &mut buf[off..off + len];
 
             // Read remote memory.
@@ -616,17 +596,19 @@ impl RemoteProcess {
             let len = unsafe { self.read_mem(ptr, addr + off as u64) }?;
             if len == 0 {
                 return Err(Errno::EFAULT);
-            } else if request.map(|req| !req.is_valid()).unwrap_or(false) {
-                return Err(Errno::ESRCH);
             }
 
             // Check for NUL-byte.
-            if let Some(nul) = memchr::memchr(0, &ptr[..len]) {
+            if let Some(nul) = memchr(0, &ptr[..len]) {
+                // SAFETY: Validate memory read as necessary.
+                // This is not possible for ptrace(2) hooks.
+                if request.map(|req| !req.is_valid()).unwrap_or(false) {
+                    return Err(Errno::ESRCH);
+                }
+
                 // Adjust to actual size up to NUL-byte.
                 off += nul;
-                buf.truncate(off);
-                buf.shrink_to_fit();
-                return Ok(buf.into());
+                return Ok(buf[..off].into());
             }
 
             off += len;
@@ -814,7 +796,8 @@ impl UNotifyEventRequest {
         SandboxGuard::Write(self.sandbox.write().unwrap_or_else(|err| err.into_inner()))
     }
 
-    /// Check SCM_RIGHTS file types at sendm{,m}sg(2), recv{,m}msg{,_time64}(2) boundaries.
+    /// Check SCM_RIGHTS file types at sendm{,m}sg(2) boundaries.
+    #[expect(clippy::cognitive_complexity)]
     pub(crate) fn check_scm_rights<Fd: AsFd>(
         &self,
         fd: Fd,
@@ -1437,15 +1420,12 @@ impl UNotifyEventRequest {
         // Open the PIDFd.
         let pid_fd = pidfd_open(self.scmpreq.pid(), PIDFD_THREAD)?;
 
-        // SAFETY:
-        // 1. Validate the PIDFd by validating the request ID if submitted.
-        // 2. EAGAIN|EINTR is handled.
-        // 3. ENOENT means child died mid-way.
-        if seccomp_notify_id_valid(self.notify_fd, self.scmpreq.id).is_err() {
-            return Err(Errno::ESRCH);
+        // SAFETY: Validate the PIDFd by validating the request ID.
+        if self.is_valid() {
+            Ok(pid_fd)
+        } else {
+            Err(Errno::ESRCH)
         }
-
-        Ok(pid_fd)
     }
 
     /// Send the request pid a signal based on the given action.
