@@ -18,6 +18,16 @@ use syd::{
     rng::fillrandom,
 };
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
@@ -65,7 +75,7 @@ syd::main! {
 
 fn flip_bit_in_file(file_name: &XPath, bit_index: usize) -> SydResult<()> {
     eprintln!("syd-bit: opening file {file_name}...");
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let mut file = OpenOptions::new().read(true).write(true).open(file_name)?;
 
     // Calculate the byte index from the bit index.
@@ -107,7 +117,7 @@ fn flip_bit_in_file(file_name: &XPath, bit_index: usize) -> SydResult<()> {
 }
 
 fn flip_random_bit_in_file(file_name: &XPath) -> SydResult<()> {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let mut file = OpenOptions::new().read(true).write(true).open(file_name)?;
 
     let file_len = file.seek(SeekFrom::End(0))?; // Get file length
