@@ -30,7 +30,7 @@ use nix::{
 use syd::{
     compat::epoll_ctl_safe,
     config::VDSO_SYSCALLS,
-    confine::{confine_mdwe, extend_ioctl},
+    confine::{confine_mdwe, confine_scmp_madvise, extend_ioctl},
     err::SydResult,
     fs::{closeexcept, set_exclusive, set_nonblock, winsize_get, winsize_set},
     ignore_signals,
@@ -395,7 +395,7 @@ fn confine(sig_fd: RawFd, dry_run: bool, print_rules: bool) -> SydResult<()> {
         // can {{dr}e,}allocate memory.
         // mmap{,2} and mprotect are further confined to disable PROT_EXEC.
         "brk",
-        "madvise",
+        //"madvise", advice are confined.
         "mremap",
         "munmap",
         // can close files.
@@ -417,6 +417,9 @@ fn confine(sig_fd: RawFd, dry_run: bool, print_rules: bool) -> SydResult<()> {
         }
     }
 
+    // Allow safe madvise(2) advice.
+    confine_scmp_madvise(&mut ctx)?;
+
     // Allow read(2) to the signal fd only.
     #[expect(clippy::disallowed_methods)]
     let syscall = ScmpSyscall::from_name("read").unwrap();
