@@ -28,13 +28,14 @@ use crate::{
     alert,
     cache::SysInterrupt,
     config::*,
-    confine::{confine_scmp_write, scmp_add_setid_rules, ExportMode},
+    confine::{confine_scmp_madvise, confine_scmp_write, scmp_add_setid_rules, ExportMode},
     cookie::{OPENAT2_COOKIE_ARG4, OPENAT2_COOKIE_ARG5},
     err::{err2no, scmp2no, SydJoinHandle, SydResult},
     error,
-    fs::{closeexcept, retry_on_eintr, seccomp_notify_id_valid, tgkill},
+    fs::{closeexcept, seccomp_notify_id_valid, tgkill},
     info,
     proc::proc_status,
+    retry::retry_on_eintr,
     sandbox::Flags,
     sigset::SydSigSet,
     workers::WorkerCache,
@@ -515,6 +516,9 @@ impl Interrupter {
         // No proc_pid_mem(5) access required here.
         confine_scmp_write(&mut ctx, None, false)?;
 
+        // Allow safe madvise(2) advice.
+        confine_scmp_madvise(&mut ctx)?;
+
         // Allow safe system calls.
         for sysname in INT_SYSCALLS.iter().chain(VDSO_SYSCALLS) {
             match ScmpSyscall::from_name(sysname) {
