@@ -30,7 +30,7 @@ use nix::{
         epoll::{Epoll, EpollCreateFlags, EpollEvent, EpollFlags},
         socket::{
             accept4, bind, listen, recvmsg, sendmsg, setsockopt, socket, sockopt::PassCred,
-            AddressFamily, Backlog, ControlMessageOwned, MsgFlags, SockFlag, SockType, UnixAddr,
+            AddressFamily, Backlog, ControlMessageOwned, SockFlag, SockType, UnixAddr,
             UnixCredentials,
         },
         stat::{umask, Mode},
@@ -41,15 +41,16 @@ use serde::{ser::SerializeMap, Serialize, Serializer};
 
 use crate::{
     alert,
-    compat::{epoll_ctl_mod_safe, epoll_ctl_safe},
+    compat::{epoll_ctl_mod_safe, epoll_ctl_safe, MsgFlags},
     config::*,
     confine::{confine_scmp_madvise, confine_scmp_write, scmp_add_setid_rules},
     err::{err2no, scmp2no, SydJoinHandle, SydResult},
-    fs::{closeexcept, peer_cred, retry_on_eintr},
+    fs::{closeexcept, peer_cred},
     hash::SydHashMap,
     info,
     landlock_policy::LandlockPolicy,
     path::{XPath, XPathBuf},
+    retry::retry_on_eintr,
     rng::duprand,
     sandbox::{Flags, Sandbox, LINE_MAX},
 };
@@ -465,7 +466,7 @@ impl IpcWorker {
         // No proc_pid_mem(5) access required here.
         confine_scmp_write(&mut ctx, None, false)?;
 
-        // Allow safe madvise(2) advices.
+        // Allow safe madvise(2) advice.
         confine_scmp_madvise(&mut ctx)?;
 
         // Allow safe system calls.
@@ -736,7 +737,7 @@ impl IpcWorker {
                     conn.stream.as_raw_fd(),
                     &mut iov,
                     Some(&mut cmsg),
-                    MsgFlags::MSG_DONTWAIT,
+                    MsgFlags::MSG_DONTWAIT.into(),
                 ) {
                     Ok(msg) if msg.bytes == 0 => {
                         // EOF on read side: if there's a pending write,
@@ -871,7 +872,7 @@ impl IpcWorker {
                     conn.stream.as_raw_fd(),
                     &iov,
                     &[],
-                    MsgFlags::MSG_DONTWAIT,
+                    MsgFlags::MSG_DONTWAIT.into(),
                     None,
                 ) {
                     Ok(0) => {
